
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "ImgTrig.h"


void plot_triangle(int x1, int y1,
                   int x2, int y2,
                   int x3, int y3,
                   int scanline(int,int,int))
{
    int y;
    if (y2 < y1 && y2 <= y3) {
        plot_triangle(x2,y2, x1,y1, x3,y3, scanline);
        return;
    }
    if (y3 < y1 && y3 <= y2) {
        plot_triangle(x3,y3, x2,y2, x1,y1, scanline);
        return;
    }
    /* (x1,y1) is the point with the minimum y-coordinate. */
    if (y3 < y2) {
        plot_triangle(x1,y1, x3,y3, x2,y2, scanline);
        return;
    }
    /* The points are now sorted by y-coordinate. */
    if (y1 > y2 || y2 > y3) {
        printf("y1=%d y2=%d y3=%d\n", y1,y2,y3);
    }
    assert(y1 <= y2 && y2 <= y3);
    /* Let's not bother with Bresenham's algorithm just yet. */
    if (y3 < 0) return;
    if (y2 < 0) goto found_y2;
    if (y1 == y2) {
        if (x1 > x2) {
            int temp = x1;
            x1 = x2;
            x2 = temp;
        }
        if (y1 == y3) {
            /* Notice that this path destroys information, rather than simply
             * reordering the points as we've been doing up to now. But since
             * we're just going to draw this one scanline, goto found_y2, and
             * then immediately return, it's okay to trash x1 and x2. */
            if (x3 < x1) x1 = x3;
            if (x3 > x2) x2 = x3;
        }
        if (scanline(y1, x1,x2)) return;
        goto found_y2;
    }
    assert(y1 < y2 && y2 <= y3);
    for (y = y1; /* true */; ++y) {
        int xleft = ((y-y1)*x2 + (y2-y)*x1) / (y2-y1);
        int xright = ((y-y1)*x3 + (y3-y)*x1) / (y3-y1);
        if (xleft > xright) {
            int temp = xleft;
            xleft = xright;
            xright = temp;
        }
        if (scanline(y, xleft,xright)) return;
        if (y == y2) {
            assert(xleft == x2 || xright == x2);
            break;
        }
    }
  found_y2:
    /* At this point, we've painted all the scanlines up to and including
     * line y2. */
    if (y2 == y3)
      return;
    assert(y1 < y3 && y2 < y3);
    for (y = y2+1; /* true */; ++y) {
        int xleft = ((y-y2)*x3 + (y3-y)*x2) / (y3-y2);
        int xright = ((y-y1)*x3 + (y3-y)*x1) / (y3-y1);
        if (xleft > xright) {
            int temp = xleft;
            xleft = xright;
            xright = temp;
        }
        if (scanline(y, xleft,xright)) return;
        if (y == y3) {
            assert(xleft == x3 && xright == x3);
            break;
        }
    }
    assert(y == y3);
    return;
}

