
/* Scan a game state from standard input.
 * Find all of player 0's "reasonable" moves from that state. A move which
 * is provably worse than (or identical to) an already-reported move is
 * not necessarily considered "reasonable", but we won't throw out any move
 * merely because it *looks* like a bad idea (e.g., sacrificing a ship for
 * no immediate gain, or destroying part of one's own homeworld).
 * Print the number of "reasonable" moves found.
 */

#include <stdio.h>
#include <vector>
#include "state.h"
#include "move.h"
#include "AllMoves.h"

int main(int argc, char **argv)
{
    int attacker = 0;
    if (argc >= 2)
      attacker = atoi(argv[1]);

    GameState st;
    st.scan(stdin);
    printf("----------------------\n");
    printf("%s", st.toString().c_str());
    printf("----------------------\n");
    std::vector<WholeMove> moves;
    findAllMoves(st, attacker, moves, false);
    int an = moves.size();
    assert(an > 0);  /* because "pass" is always a legal move */
    if (an == 1) {
        printf("Player %d has only one reasonable move:\n", attacker);
    } else {
        printf("Player %d has %d reasonable moves:\n", attacker, an);
    }
    for (int i=0; i < (int)moves.size(); ++i) {
        printf("%d.  %s\n", i+1, moves[i].toString().c_str());
    }
    return 0;
}
