
/* Scan a game state from standard input.
 * Player 0 is trying to wriggle out of a losing position.
 * 1. Compute all of player 0's possible moves; weed out the ones that
 * immediately lose; and present the rest to the human operator.
 * 2. The human picks one path to explore, or backtracks up the tree.
 * 3. The human enters what he thinks is the best move for player 1 on that path.
 * 4. Return to step 1.
 */

#include <stdio.h>
#include <vector>
#include <stack>
#include "state.h"
#include "move.h"
#include "AllMoves.h"
#include "ApplyMove.h"
#include "AlphaBeta.h"

static char buffer[1000];


int main(int argc, char **argv)
{
    int loser = 0;
    if (argc >= 2)
      loser = atoi(argv[1]);

    GameState st;
    st.scan(stdin);

    std::stack<GameState> history;

  pursue_a_hypothetical:
    history.push(st);
    printf("----------------------\n");
    printf("%s", st.toString().c_str());
    printf("----------------------\n");
    printf("It is the losing player %d's turn.\n", loser);
    /* Compute all of the losing player's moves. */
    std::vector<WholeMove> allmoves;
    findAllMoves_usualcase(st, loser, &allmoves);
    assert(!allmoves.empty());
    /* Weed out the immediately losing ones. */
    std::vector<WholeMove> nonlosingmoves;
    for (int i=0; i < (int)allmoves.size(); ++i) {
        GameState newst = st;
        ApplyMove::or_die(newst, loser, allmoves[i]);
        std::vector<WholeMove> winning_responses;
        findAllMoves(newst, 1-loser, &winning_responses, NULL);
        if (winning_responses.empty())
          nonlosingmoves.push_back(allmoves[i]);
    }
    if (nonlosingmoves.empty()) {
        printf("Player %d must necessarily lose at this point.\n", loser);
        /* The winning player's last move was perfect; but maybe the
         * losing player should have pursued a different hypothetical
         * two turns ago. Backtrack and choose a different hypothetical
         * to pursue. */
        assert(!history.empty());
        history.pop();
        if (history.empty()) {
            printf("You needed me to tell you that?\n");
            exit(EXIT_FAILURE);
        } else {
            printf("Backtracking to choose a different hypothetical for the loser...\n");
            st = history.top();
            history.pop();
            goto pursue_a_hypothetical;
        }
    }
    /* The losing player is not completely cornered yet. */
    printf("Losing player %d, has %d possible moves:\n",
        loser, (int)nonlosingmoves.size());
    for (int i=0; i < (int)nonlosingmoves.size(); ++i) {
        printf("%d. %s\n", i+1, nonlosingmoves[i].toString().c_str());
    }
    printf("Pursue which hypothetical? .> "); fflush(stdout);
    gets(buffer);
    int which = atoi(buffer);
    if (which <= 0 || (int)nonlosingmoves.size() < which) {
        /* Backtrack and choose a different move for the winning player
         * last turn. */
        assert(!history.empty());
        history.pop();
        if (history.empty()) {
            printf("You can't backtrack from the top of the tree!\n");
            goto pursue_a_hypothetical;
        } else {
            st = history.top();
            history.pop();
            goto get_winners_move;
        }
    }
    {
    /* The loser selected a hypothetical to pursue. */
    WholeMove &pursuedmove = nonlosingmoves[which-1];
    ApplyMove::or_die(st, loser, pursuedmove);
    }

  get_winners_move:
    {
    printf("----------------------\n");
    printf("%s", st.toString().c_str());
    printf("----------------------\n");
    printf("It is winning player %d's turn.\n", 1-loser);
    printf("Move? .> "); fflush(stdout);
    gets(buffer);
    WholeMove winners_move(buffer);
    ApplyMove::or_die(st, 1-loser, winners_move);
    goto pursue_a_hypothetical;
    }

    /*NOTREACHED*/    
    return 0;
}

